/***************************************************************************
 *            basicwindow.h
 *
 *  Fri Jan 21 23:00:12 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#ifndef BASICWINDOW_H
#define BASICWINDOW_H

#include <gtk/gtk.h>
#include <iostream>

#include <gdk/gdkkeysyms.h>
#include "imuserdaemon.h"
#include "settings.h"
#include "utilities.h"


enum containerModes
{
	CM_TYPE,
	CM_USER,
	CM_MESSAGEWINDOW
};

class windowContainer;


class basicWindow
{
	public:
		userEventType	wType;	
		IMEventManager	*manager;
		GtkWidget		*content,
						*tabLabel;
		gint			sizeX,
						sizeY;
		GtkAccelGroup	*accelerators;
	
		basicWindow();
		virtual ~basicWindow();
	
		// creates a basic window
		gboolean createWindow();
		
		// called when a window looses focus
		virtual void windowDeactivated();
		// called when a window gets focus
		virtual void windowActivated();
	
		// gives the focus to that window
		virtual void focusWindow();
	
		// sets the window title
		void setWindowTitle(gchar* _title);
				
		// sets the windows size
		void setWindowSize(gint _sizeX, gint _sizeY);
	
		// callback function called by the buddy
		virtual gboolean eventCallback(gint command, gint lastResult, gpointer info);
	
		// callback function for setting changes
		virtual void settingsChangedCallback(gchar* sname, GList *values);
	
		// destroys the window 
		gboolean destroyWindow();
		
		// closes the window (is called when the window is closed manually)
		virtual void closeWindow();
		
		// adds a listener to the settings
		virtual void addSettingsListener(gchar* sname);
		
		// allocates extra space on the right side of the window
		virtual void allocateExtraSpace(gint toAllocate);
		
		// deallocates the extra space on the right side
		virtual void deallocateExtraSpace();

		void setDestroyCallback(IMEventCallback _dc, gpointer _dcd);

		static gboolean cb_eventCallback(userEventType type, gint command, gint lastResult, gpointer info, basicWindow* self);

	
	protected:
		
		GtkWidget	*window;
		gchar		*title;
		gboolean	hasFocus;
		gboolean	hasSettingsEntries;
		gint		extraSpace;
	
		IMEventCallback	destroyCallback;
		gpointer		destroyCallbackData;
		
		// creates the real window content
		virtual GtkWidget* createWindowContent();
		virtual void	   destroyWindowContent();

		// finds the container the window is in
		gpointer getWindowContainer();
	
		// sets the container widget
		virtual GtkWidget* createContainerTabWidget();
	
		// what to do when ESC was pressed
		virtual void escPressed();

	
		// callbacks
		static gboolean cb_destroyWindow(basicWindow *bw);
		static gboolean cb_windowFocusOut(basicWindow *bw);
		static gboolean cb_windowFocusIn(basicWindow *bw);
		static void		cb_escPressed(basicWindow* self);
	
		static void cb_settingsChangedCallback(gchar* sname, GList *values, basicWindow* self);


	friend class windowContainer;
};




class windowContainer
{
	public:
		gint cType;
		GtkWidget* window;	
		GtkAccelGroup	*accelerators;
	
		windowContainer();
		~windowContainer();
	
		void addWindow(basicWindow* toAdd);
		void removeWindow(basicWindow* toRemove, GtkWidget* rcontent);
		void focusWindow(basicWindow* toFocus);
		void allocateExtraSpace(gint toAllocate);
		void deallocateExtraSpace();
		void updateWindowTitle();
	
		gboolean containsWindow(basicWindow* toLook);
	
	private:
		GtkWidget* noteBook;	
		gint	   extraSpace,
				   extraSpaceCount;
	
		GList *children;
	
		static void cb_destroyContainer(windowContainer* wc);
		static gboolean cb_containerFocusOut(windowContainer* wc);
		static gboolean cb_containerFocusIn(windowContainer* wc);
		static void cb_switchWindow(GtkWidget *w, GtkNotebookPage *page, guint newTab, windowContainer* wc);
		static void cb_closeWindowButtonClicked(GtkWidget *button, windowContainer* wc);
		static void cb_closeActiveTab(windowContainer* self);
		static void	cb_escPressed(windowContainer* self);
	
		static void cb_accelLeftPressed(windowContainer* self);
		static void cb_accelRightPressed(windowContainer* self);
};


GList* basicWindow_getAllWindows();


#endif
