/***************************************************************************
 *            requestdialog.cpp
 *
 *  Thu Feb 10 00:11:55 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

#include "requestdialog.h"



requestDialog::requestDialog(gboolean			_isServer, 
					 		 GtkWidget*			_specialWidget, 
					 		 gchar*				_topicText,
					 		 gchar* 			_infoText, 
					 		 gchar* 			_commentText, 
							 gchar*				_logoImage,
					 		 requestFunction 	_clickNotify, 
					 		 gpointer 			_data)
{
	isServer = _isServer;
	specialWidget = _specialWidget;
	_topicText   ? topicText   = g_strdup_printf("<b>%s</b>", _topicText) 	: topicText = NULL;
	_infoText    ? infoText    = g_strdup_printf("<small>%s</small>", _infoText)  	: infoText  = NULL;
	_commentText ? commentText = g_strdup(_commentText) : commentText = NULL; 

	if (_logoImage)
		logoImage = g_strdup_printf("%sicqnd/%s", SHARE_DIR, _logoImage);
	else
		logoImage = NULL;
	
	preTitle = NULL;
	
	requestName = NULL;
	
	clickNotify = _clickNotify;
	data		= _data;
	
	sendAni		= NULL;
	sendButton	= NULL;
	
	isInfoWindowOnly = FALSE;
	hasCommentbox = TRUE;
	
}

requestDialog::~requestDialog()
{
	if (topicText) g_free(topicText);
	if (infoText) g_free(infoText);
	if (commentText) g_free(commentText);
	if (requestName) g_free(requestName);
	if (preTitle) g_free(preTitle);
	if (logoImage) g_free(logoImage);
}


void requestDialog::setInfoWindowOnly(gboolean setting)
{
	isInfoWindowOnly = setting;
}


void requestDialog::setTopicInformation(gchar *_topicText, gchar* _infoText)
{
	if (topicText) g_free(topicText);
	if (infoText) g_free(infoText);
	_topicText   ? topicText   = g_strdup(_topicText) 	: topicText = NULL;
	_infoText    ? infoText    = g_strdup(_infoText)  	: infoText  = NULL;
		
}



void requestDialog::setComment(gchar* _commentText)
{
	if (commentText) g_free(commentText);
	_commentText ? commentText = g_strdup(_commentText) : commentText = NULL; 
}


void requestDialog::setRequestName(gchar* _requestName)
{
	if (requestName) g_free(requestName);
	_requestName ? requestName = g_strdup(_requestName) : requestName = NULL;
}


GtkWidget* requestDialog::createWindowContent()
{
	GtkWidget	*label1,
				*label2,
				*hbox1,
				*hbox2,
				*vbox1,
				*vbox2,
				*alignment1,
				*notebook,
				*frame1,
				*mainVBox,
				*infoButton,
				*historyButton,
				*img,
				*paned;
	
	gchar		*introText,
				*text;
	
	
	// create info button and history button
	infoButton = u_createStockImageButton(ICON_INFO);
	g_signal_connect_swapped(infoButton, "clicked", G_CALLBACK(this->cb_infoButtonClicked), this);
	historyButton = u_createStockImageButton(ICON_HISTORY);
	g_signal_connect_swapped(historyButton, "clicked", G_CALLBACK(this->cb_historyButtonClicked), this);

	hbox1 = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox1), infoButton, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), historyButton, FALSE, FALSE, 0);
	
	alignment1 = gtk_alignment_new(0, 0, 0, 0);
	gtk_container_add(GTK_CONTAINER(alignment1), hbox1);
	
	
	// create the labels
	introText = g_strdup_printf(topicText, manager->info->alias);
	label1 = gtk_label_new(introText);
	gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label1), 0, 0);
	
	label2 = gtk_label_new(infoText);
	gtk_widget_set_size_request(label2, 270, -1);
	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
	gtk_label_set_line_wrap(GTK_LABEL(label2), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label2), 0, 0);
	
	vbox1 = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox1), label1, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), label2, TRUE, TRUE, 0);
	
	// create the picture
	if (logoImage)
		img = gtk_image_new_from_file(logoImage);
	else
		img = NULL;
	
	
	// add everything to the main intro box
	hbox2 = gtk_hbox_new(FALSE, 5);
	if (img)
		gtk_box_pack_start(GTK_BOX(hbox2), img, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox2), vbox1, FALSE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(hbox2), alignment1, FALSE, TRUE, 0);
	
	
	
	// create the comment box
	if (hasCommentbox)
	{
		frame1 = u_createTextView(&comment);
		gtk_text_view_set_editable(GTK_TEXT_VIEW(comment), isServer);
		if (commentText)
			gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(comment)),
									 commentText,
									 strlen(commentText) );	
	
		// create the comment label
		label1 = gtk_label_new(tr("<b>Comment</b>"));
		gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
		gtk_misc_set_alignment(GTK_MISC(label1), 0, 0);

		vbox1 = gtk_vbox_new(FALSE, 2);
		gtk_box_pack_start(GTK_BOX(vbox1), label1, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(vbox1), frame1, TRUE, TRUE, 0);	

		paned = gtk_vpaned_new();
		if (specialWidget)
			gtk_paned_pack1(GTK_PANED(paned), specialWidget, TRUE, FALSE);
		gtk_paned_pack2(GTK_PANED(paned), vbox1, FALSE, TRUE);
	} else
		if (specialWidget)
			paned = specialWidget;
		else
			paned = NULL;
	
	
	// add everything into a big vbox
	vbox2 = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 5);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox2, FALSE, TRUE, 0);
	if (paned)
		gtk_box_pack_start(GTK_BOX(vbox2), paned, TRUE, TRUE, 0);
	
	
	// create a notebook around it
	notebook = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox2, NULL);
	
	// add everything into a big vbox + button bar
	mainVBox = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER(mainVBox), 10);
	gtk_box_pack_start(GTK_BOX(mainVBox), notebook, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(mainVBox), createButtonbar(), FALSE, TRUE, 0);
	
	if (preTitle)
	{
		text = g_strdup_printf(preTitle, manager->info->alias);
		setWindowTitle(text);
		g_free(text);		
	}
	
	if (!specialWidget)
		setWindowSize(300, 300);
	else
		setWindowSize(300, 450);
	
	
	return mainVBox;
	
	
	/*
	GtkWidget	*vbox1,
				*vbox2,
				*hbox1, 
				*label,
				*label2,
				*frame,
				*mainVbox,
				*infoButton,
				*historyButton,
				*alignment,
				*alignment2;
		
	gchar		*text, 
				*text2;	
	gboolean	useRealName;
	
	
	// create headline (info-button, history-button, name, "has a request")
	infoButton = u_createStockImageButton(ICON_INFO);
	g_signal_connect_swapped(infoButton, "clicked", G_CALLBACK(this->cb_infoButtonClicked), this);
	historyButton = u_createStockImageButton(ICON_HISTORY);
	g_signal_connect_swapped(historyButton, "clicked", G_CALLBACK(this->cb_historyButtonClicked), this);



	label = gtk_label_new("");
	settings_getSettings()->getProperties("contactlist", "showRealNames", &useRealName, NULL);
	if (useRealName)
		text2 = uu_getRealNameOrAlias(manager->info, FALSE);
	else
		text2 = g_strdup(manager->info->alias);
	text = g_strdup_printf(topicText, text2);
	gtk_label_set_markup(GTK_LABEL(label), text);
	g_free(text);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	
	if (preTitle)
	{
		text = g_strdup_printf(preTitle, text2);
		setWindowTitle(text);
		g_free(text);		
	}
	
	g_free(text2);
	
	hbox1 = gtk_hbox_new(FALSE,1);
	gtk_box_pack_start(GTK_BOX(hbox1), infoButton, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), historyButton, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox1), label, TRUE, TRUE, 2);
		
	
	// create the intro label
	label = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(label), infoText);
	
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
		

	
	vbox1 = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, TRUE, 0);
	if (specialWidget)
		gtk_box_pack_start(GTK_BOX(vbox1), specialWidget, TRUE, TRUE, 3);
	
	alignment2 = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment2), 0, 0, 20, 8);
	gtk_container_add(GTK_CONTAINER(alignment2), vbox1);
			
	
	
	// create the comment box	
	frame = u_createTextView(&comment);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(comment), isServer);
	if (commentText)
		gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(comment)),
								 commentText,
								 strlen(commentText) );	
	
	alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 20, 8);
	

	// create a special text when the request is local
	if (isServer)
	{
		// create a vbox containing the description instruction + comment box
		label2 = gtk_label_new("");
		gtk_label_set_markup(GTK_LABEL(label2), tr("<small>Please enter a short comment or a description here</small>"));
		gtk_label_set_line_wrap(GTK_LABEL(label2), TRUE);
		gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);
	
		vbox2 = gtk_vbox_new(FALSE, 4);
		gtk_box_pack_start(GTK_BOX(vbox2), label2, FALSE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, TRUE, 0);
		gtk_container_add(GTK_CONTAINER(alignment), vbox2);
		
	} else
		gtk_container_add(GTK_CONTAINER(alignment), frame);

	
	
	// create a "comment" label
	label2 = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(label2), tr("<b>Comment</b>"));
	gtk_label_set_line_wrap(GTK_LABEL(label2), TRUE);
	gtk_misc_set_alignment(GTK_MISC(label2), 0.0f, 0.0f);
	gtk_misc_set_padding(GTK_MISC(label2), 6, 0);

	
	
	// pack everything into a vbox
	mainVbox = gtk_vbox_new(FALSE,4);
	gtk_container_set_border_width(GTK_CONTAINER(mainVbox), 6);
	gtk_box_pack_start(GTK_BOX(mainVbox), hbox1, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(mainVbox), alignment2, TRUE, TRUE, 0);	
	gtk_box_pack_start(GTK_BOX(mainVbox), label2, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(mainVbox), alignment, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(mainVbox), gtk_hseparator_new(), FALSE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(mainVbox), createButtonbar(), FALSE, TRUE,0);

		
	return mainVbox;	*/	
}



void requestDialog::destroyWindowContent()
{
	if (sendAni)
	{
		sendAni->stop();
		delete sendAni;
	}
}




void requestDialog::closeWindow()
{
	basicWindow::closeWindow();
	
	if (clickNotify)
		clickNotify(REQUEST_CLOSED, NULL, data);		
}



GtkWidget*	requestDialog::createButtonbar()
{
	GtkWidget	*hbox,
				*hbox2,
				*alignment,
				*acceptButton,
				*refuseButton1,
				*refuseButton2;
	gchar		*txt, *txt2;
	
	
	if (isInfoWindowOnly)
	{
		closeButton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
		g_signal_connect_swapped(closeButton, "clicked", G_CALLBACK(this->cb_closeButtonClicked), this);

		hbox = gtk_hbox_new(FALSE, 3);		
		gtk_box_pack_end(GTK_BOX(hbox), closeButton, FALSE, FALSE, 0);

		return hbox;		
	}
	
	
	if (isServer)
	{	
		sendAni = new processingAni();
		sendAni->setupAnimation(i_getIcons()->images.searchingAni, 22, 22);
	
	
		// create the send button
		hbox2 = gtk_hbox_new(FALSE,2);

		if (requestName)
			txt = requestName;
		else
			txt = tr_NOOP("Request");
		
		txt2 = g_strdup_printf(tr_NOOP("_Send %s"), txt);
		sendButton = gtk_button_new_with_mnemonic(tr(txt2));
		g_free(txt2);
		g_signal_connect_swapped(sendButton, "clicked", G_CALLBACK(this->cb_sendButtonClicked), this);
	
		closeButton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
		g_signal_connect_swapped(closeButton, "clicked", G_CALLBACK(this->cb_closeButtonClicked), this);
	
		txt2 = g_strdup_printf(tr_NOOP("Send %s via server"), txt);
		serverCheckbox = gtk_check_button_new_with_label(tr(txt2));
		g_free(txt2);
		g_signal_connect_swapped(serverCheckbox, "clicked", G_CALLBACK(this->cb_sendServerButtonClicked), this);
	
		alignment= gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
		gtk_container_add(GTK_CONTAINER(alignment), serverCheckbox);	
	
		hbox = gtk_hbox_new(FALSE, 3);
		gtk_box_pack_start(GTK_BOX(hbox), sendAni->widget, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(hbox), alignment, TRUE, TRUE, 2);
		gtk_box_pack_end(GTK_BOX(hbox), closeButton, FALSE, FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), sendButton, FALSE, FALSE, 2);
	} else
	{
		acceptButton = u_createTextStockImageButton(tr("Accept"), GTK_STOCK_YES);
		g_signal_connect_swapped(acceptButton, "clicked", G_CALLBACK(this->cb_sendButtonClicked), this);
		refuseButton1 = u_createTextStockImageButton(tr("Refuse with substantiation"), GTK_STOCK_NO);
		g_signal_connect_swapped(refuseButton1, "clicked", G_CALLBACK(this->cb_refuseWithReasonButtonClicked), this);
		refuseButton2 = u_createTextStockImageButton(tr("Refuse"), GTK_STOCK_NO);
		g_signal_connect_swapped(refuseButton2, "clicked", G_CALLBACK(this->cb_refuseButtonClicked), this);
	
		alignment = gtk_alignment_new(0.0f, 0.0f, 1.0f, 1.0f);
		gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 0, 10);
		gtk_container_add(GTK_CONTAINER(alignment), acceptButton);
	
		hbox = gtk_hbox_new(FALSE, 3);
		gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), refuseButton2, FALSE, FALSE, 0);
		gtk_box_pack_end(GTK_BOX(hbox), refuseButton1, FALSE, FALSE, 0);	
	}
	
	return hbox;
}



gboolean requestDialog::eventCallback(gint command, gint lastResult, gpointer info)
{
	switch(command)
	{
		// the manager was installed
		case EC_INSTALLED:
		// the connection state has changed - update the label
		case UC_SENDSERVER:
			if (isServer)
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(serverCheckbox), manager->info->sendViaServer);
			break;
	}
	return TRUE;	
}



gchar* requestDialog::askForReason(gchar* headline, gchar* hltext)
{
	GtkWidget	*label,
				*vbox,
				*reasonWindow,
				*tv;
	
	GtkTextBuffer *tbuf;
	GtkTextIter iter1, iter2;
	
	gchar		*txt,
				*charData;
	
	gint		res;
	
	reasonWindow = gtk_dialog_new_with_buttons(headline, GTK_WINDOW(window), 
											   (GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
                                               GTK_STOCK_OK,
                                               GTK_RESPONSE_ACCEPT,
                                               GTK_STOCK_CANCEL,
                                               GTK_RESPONSE_REJECT,
											   NULL);
	
	label = gtk_label_new(hltext);
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	
	
	// create the text entry field
	vbox = gtk_vbox_new(FALSE, 4);	
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), u_createTextView(&tv), TRUE, TRUE, 0);
	gtk_widget_show_all(vbox);
	
	gtk_widget_set_size_request(tv, -1, 200);
	
	
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(reasonWindow)->vbox), vbox);
	
	res = gtk_dialog_run(GTK_DIALOG(reasonWindow));
	
	
	if (res == GTK_RESPONSE_ACCEPT)
	{
		tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(tv));
		gtk_text_buffer_get_bounds(tbuf, &iter1, &iter2);
		txt = gtk_text_buffer_get_text(tbuf, &iter1, &iter2, FALSE);
		// FIXME: convert to some charset
		charData = convertFromSystemCharset(txt, manager->info->charset);
		g_free(txt);
		
		// ((IMAdmissionManager*)manager)->answerRemoteRequest(FALSE, charData);
		
		gtk_widget_destroy(reasonWindow);
	
	} else
	{
		gtk_widget_destroy(reasonWindow);
		charData=g_strdup("");
	}
	
	return charData;
}




void requestDialog::waitForRequestAnswer()
{
	if (!isServer)
		return;
	gtk_widget_set_sensitive(sendButton, FALSE);
	gtk_widget_set_sensitive(serverCheckbox, FALSE);
	
	sendAni->play();
	
	gtk_button_set_label(GTK_BUTTON(closeButton), GTK_STOCK_CANCEL);
	gtk_button_set_use_stock(GTK_BUTTON(closeButton), TRUE);
}



void requestDialog::stopWaitForRequestAnswer()
{
	if (!isServer)
		return;
	
	gtk_widget_set_sensitive(sendButton, TRUE);
	gtk_widget_set_sensitive(serverCheckbox, TRUE);
	
	sendAni->stop();
	
	gtk_button_set_label(GTK_BUTTON(closeButton), GTK_STOCK_CLOSE);
	gtk_button_set_use_stock(GTK_BUTTON(closeButton), TRUE);
}



void requestDialog::escPressed()
{
	cb_closeButtonClicked(this);
}



void requestDialog::cb_sendButtonClicked(requestDialog* rd)
{

	GtkTextIter 	startIter, 
					endIter;
	gchar			*commentText;
	GtkTextBuffer	*buf;
	
	gchar*			tmp;
	
	commentText = NULL;
	
	// we are in server mode -> read the comment box content
	if (rd->isServer)
	{
		if (rd->hasCommentbox)
		{
			buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(rd->comment));
			gtk_text_buffer_get_bounds(buf, &startIter, &endIter);
	
			tmp = gtk_text_buffer_get_text(buf, &startIter, &endIter, FALSE);
	
			commentText = convertFromSystemCharset(tmp, rd->manager->info->charset);
			
			g_free(tmp);
		}			
		else
			commentText = NULL;
	}
	
	if (rd->clickNotify)
		rd->clickNotify(REQUEST_ACCEPTED, commentText, rd->data);
	
	if (rd->isServer && rd->hasCommentbox)
		g_free(commentText);		
}



void requestDialog::cb_refuseButtonClicked(requestDialog* rd)
{
	gchar* charData;
	
	charData = "";
	
	if (rd->clickNotify)
		rd->clickNotify(REQUEST_REFUSED, charData, rd->data);
}



void requestDialog::cb_refuseWithReasonButtonClicked(requestDialog* rd)
{
	gchar *charData;
	charData = rd->askForReason(tr("Refuse request"), tr("Please enter the reason for the refusal"));
	if (rd->clickNotify)
		rd->clickNotify(REQUEST_REFUSED, charData, rd->data);	
	g_free(charData);
}



void requestDialog::cb_closeButtonClicked(requestDialog* rd)
{
	if (rd->sendButton && !GTK_WIDGET_IS_SENSITIVE(rd->sendButton))
	{
		if (rd->clickNotify)
			rd->clickNotify(REQUEST_CANCELLED, NULL, rd->data);
	} else
	{
		if (rd->clickNotify)
			rd->clickNotify(REQUEST_CLOSED, NULL, rd->data);
		
		rd->destroyWindow();
		delete rd;
	}
}



void requestDialog::cb_sendServerButtonClicked(requestDialog* rd)
{
	rd->manager->setSendViaServer(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rd->serverCheckbox)));	
}



void  requestDialog::cb_infoButtonClicked(requestDialog* rd)
{
	rd->manager->callUser(UE_USERINFO, NULL);
}



void  requestDialog::cb_historyButtonClicked(requestDialog* rd)
{
	rd->manager->callUser(UE_HISTORY, NULL);
}
